@@ -1,6 +1,7 @@
-/* $Id: thunar-uca-provider.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,7 +23,10 @@
 #include <config.h>
 #endif
 
-#include <thunar-vfs/thunar-vfs.h>
+#include <gio/gio.h>
+
+#include <libxfce4util/libxfce4util.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <thunar-uca/thunar-uca-chooser.h>
 #include <thunar-uca/thunar-uca-context.h>
@@ -32,10 +36,8 @@
 
 
 
-static void   thunar_uca_provider_class_init                (ThunarUcaProviderClass           *klass);
 static void   thunar_uca_provider_menu_provider_init        (ThunarxMenuProviderIface         *iface);
 static void   thunar_uca_provider_preferences_provider_init (ThunarxPreferencesProviderIface  *iface);
-static void   thunar_uca_provider_init                      (ThunarUcaProvider                *uca_provider);
 static void   thunar_uca_provider_finalize                  (GObject                          *object);
 static GList *thunar_uca_provider_get_actions               (ThunarxPreferencesProvider       *preferences_provider,
                                                              GtkWidget                        *window);
@@ -47,10 +49,10 @@ static GList *thunar_uca_provider_get_folder_actions        (ThunarxMenuProvider
                                                              ThunarxFileInfo                  *folder);
 static void   thunar_uca_provider_activated                 (ThunarUcaProvider                *uca_provider,
                                                              GtkAction                        *action);
-static void   thunar_uca_provider_child_watch               (GPid                              pid,
-                                                             gint                              status,
-                                                             gpointer                          user_data);
-static void   thunar_uca_provider_child_watch_destroy       (gpointer                          user_data);
+static void   thunar_uca_provider_child_watch               (ThunarUcaProvider                *uca_provider,
+                                                             gint                              exit_status);
+static void   thunar_uca_provider_child_watch_destroy       (gpointer                          user_data,
+                                                             GClosure                         *closure);
 
 
 
@@ -71,7 +73,7 @@ struct _ThunarUcaProvider
    * child process has terminated.
    */
   gchar          *child_watch_path;
-  gint            child_watch_id;
+  GClosure       *child_watch;
 };
 
 
@@ -133,10 +135,6 @@ thunar_uca_provider_init (ThunarUcaProvider *uca_provider)
 
   /* grab a reference on the default model */
   uca_provider->model = thunar_uca_model_get_default ();
-
-  /* initialize child watch support */
-  uca_provider->child_watch_path = NULL;
-  uca_provider->child_watch_id = -1;
 }
 
 
@@ -145,18 +143,9 @@ static void
 thunar_uca_provider_finalize (GObject *object)
 {
   ThunarUcaProvider *uca_provider = THUNAR_UCA_PROVIDER (object);
-  GSource           *source;
 
   /* give up maintaince of any pending child watch */
-  if (G_UNLIKELY (uca_provider->child_watch_id >= 0))
-    {
-      /* reset the callback function to g_spawn_close_pid() so the plugin can be
-       * safely unloaded and the child will still not become a zombie afterwards.
-       * This also resets the child_watch_id and child_watch_path properties.
-       */
-      source = g_main_context_find_source_by_id (NULL, uca_provider->child_watch_id);
-      g_source_set_callback (source, (GSourceFunc) g_spawn_close_pid, NULL, NULL);
-    }
+  thunar_uca_provider_child_watch_destroy (uca_provider, NULL);
 
   /* drop our reference on the model */
   g_object_unref (G_OBJECT (uca_provider->model));
@@ -208,7 +197,7 @@ thunar_uca_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
   GList               *actions = NULL;
   GList               *paths;
   GList               *lp;
-  gchar               *stock_id;
+  gchar               *icon_name;
   gchar               *tooltip;
   gchar               *label;
   gchar               *name;
@@ -222,7 +211,7 @@ thunar_uca_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
           /* determine the label, tooltip and stock-id for the item */
           gtk_tree_model_get (GTK_TREE_MODEL (uca_provider->model), &iter,
                               THUNAR_UCA_MODEL_COLUMN_NAME, &label,
-                              THUNAR_UCA_MODEL_COLUMN_STOCK_ID, &stock_id,
+                              THUNAR_UCA_MODEL_COLUMN_ICON, &icon_name,
                               THUNAR_UCA_MODEL_COLUMN_DESCRIPTION, &tooltip,
                               -1);
 
@@ -230,7 +219,8 @@ thunar_uca_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
           name = g_strdup_printf ("ThunarUca::action-%d", ++uca_provider->last_action_id);
 
           /* create the new action with the given parameters */
-          action = gtk_action_new (name, label, tooltip, stock_id);
+          action = gtk_action_new (name, label, tooltip, NULL);
+          gtk_action_set_icon_name (action, icon_name);
 
           /* grab a tree row reference on the given path */
           row = gtk_tree_row_reference_new (GTK_TREE_MODEL (uca_provider->model), lp->data);
@@ -253,7 +243,7 @@ thunar_uca_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
           actions = g_list_prepend (actions, action);
 
           /* cleanup */
-          g_free (stock_id);
+          g_free (icon_name);
           g_free (tooltip);
           g_free (label);
           g_free (name);
@@ -306,7 +296,6 @@ thunar_uca_provider_activated (ThunarUcaProvider *uca_provider,
   GtkWidget           *dialog;
   GtkWidget           *window;
   gboolean             succeed;
-  GSource             *source;
   GError              *error = NULL;
   GList               *files;
   gchar              **argv;
@@ -315,7 +304,9 @@ thunar_uca_provider_activated (ThunarUcaProvider *uca_provider,
   gchar               *label;
   gchar               *uri;
   gint                 argc;
-  gint                 pid;
+  gchar               *icon_name = NULL;
+  gboolean             startup_notify;
+  GClosure            *child_watch;
 
   g_return_if_fail (THUNAR_UCA_IS_PROVIDER (uca_provider));
   g_return_if_fail (GTK_IS_ACTION (action));
@@ -339,6 +330,12 @@ thunar_uca_provider_activated (ThunarUcaProvider *uca_provider,
   succeed = thunar_uca_model_parse_argv (uca_provider->model, &iter, files, &argc, &argv, &error);
   if (G_LIKELY (succeed))
     {
+      /* get the icon name and whether startup notification is active */
+      gtk_tree_model_get (GTK_TREE_MODEL (uca_provider->model), &iter,
+                          THUNAR_UCA_MODEL_COLUMN_ICON, &icon_name,
+                          THUNAR_UCA_MODEL_COLUMN_STARTUP_NOTIFY, &startup_notify,
+                          -1);
+
       /* determine the working from the first file */
       if (G_LIKELY (files != NULL))
         {
@@ -362,36 +359,45 @@ thunar_uca_provider_activated (ThunarUcaProvider *uca_provider,
           g_free (uri);
         }
 
+      /* build closre for child watch */
+      child_watch = g_cclosure_new_swap (G_CALLBACK (thunar_uca_provider_child_watch),
+                                         uca_provider, thunar_uca_provider_child_watch_destroy);
+      g_closure_ref (child_watch);
+      g_closure_sink (child_watch);
+
       /* spawn the command on the window's screen */
-      succeed = gdk_spawn_on_screen (gtk_widget_get_screen (GTK_WIDGET (window)), working_directory,
-                                     argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH,
-                                     NULL, NULL, &pid, &error);
+      succeed = xfce_spawn_on_screen_with_child_watch (gtk_widget_get_screen (GTK_WIDGET (window)),
+                                                       working_directory, argv, NULL,
+                                                       G_SPAWN_SEARCH_PATH,
+                                                       startup_notify,
+                                                       gtk_get_current_event_time (),
+                                                       icon_name,
+                                                       child_watch,
+                                                       &error);
 
       /* check if we succeed */
       if (G_LIKELY (succeed))
         {
-          /* check if we already have a child watch */
-          if (G_UNLIKELY (uca_provider->child_watch_id >= 0))
-            {
-              /* reset the callback function to g_spawn_close_pid() so the plugin can be
-               * safely unloaded and the child will still not become a zombie afterwards.
-               */
-              source = g_main_context_find_source_by_id (NULL, uca_provider->child_watch_id);
-              g_source_set_callback (source, (GSourceFunc) g_spawn_close_pid, NULL, NULL);
-            }
+          /* release existing child watch */
+          thunar_uca_provider_child_watch_destroy (uca_provider, NULL);
 
-          /* schedule the new child watch */
-          uca_provider->child_watch_id = g_child_watch_add_full (G_PRIORITY_LOW, pid, thunar_uca_provider_child_watch,
-                                                                 uca_provider, thunar_uca_provider_child_watch_destroy);
+          /* set new closure */
+          uca_provider->child_watch = child_watch;
 
           /* take over ownership of the working directory as child watch path */
           uca_provider->child_watch_path = working_directory;
           working_directory = NULL;
         }
+      else
+        {
+          /* spawn failed, release watch */
+          g_closure_unref (child_watch);
+        }
 
       /* cleanup */
       g_free (working_directory);
       g_strfreev (argv);
+      g_free (icon_name);
     }
 
   /* present error message to the user */
@@ -415,35 +421,37 @@ thunar_uca_provider_activated (ThunarUcaProvider *uca_provider,
 
 
 static void
-thunar_uca_provider_child_watch (GPid     pid,
-                                 gint     status,
-                                 gpointer user_data)
+thunar_uca_provider_child_watch (ThunarUcaProvider *uca_provider,
+                                 gint               exit_status)
+
 {
-  ThunarUcaProvider *uca_provider = THUNAR_UCA_PROVIDER (user_data);
-  ThunarVfsMonitor  *monitor;
-  ThunarVfsPath     *path;
+  GFileMonitor *monitor;
+  GFile        *file;
+
+  g_return_if_fail (THUNAR_UCA_IS_PROVIDER (uca_provider));
 
   GDK_THREADS_ENTER ();
 
   /* verify that we still have a valid child_watch_path */
   if (G_LIKELY (uca_provider->child_watch_path != NULL))
     {
-      /* determine the corresponding ThunarVfsPath */
-      path = thunar_vfs_path_new (uca_provider->child_watch_path, NULL);
-      if (G_LIKELY (path != NULL))
-        {
-          /* schedule a changed notification on the path */
-          monitor = thunar_vfs_monitor_get_default ();
-          thunar_vfs_monitor_feed (monitor, THUNAR_VFS_MONITOR_EVENT_CHANGED, path);
-          g_object_unref (G_OBJECT (monitor));
+      /* determine the corresponding file */
+      file = g_file_new_for_path (uca_provider->child_watch_path);
 
-          /* release the ThunarVfsPath */
-          thunar_vfs_path_unref (path);
+      /* schedule a changed notification on the path */
+      monitor = g_file_monitor (file, G_FILE_MONITOR_NONE, NULL, NULL);
+
+      if (monitor != NULL)
+        {
+          g_file_monitor_emit_event (monitor, file, file, G_FILE_MONITOR_EVENT_CHANGED);
+          g_object_unref (monitor);
         }
+
+      /* release the file */
+      g_object_unref (file);
     }
 
-  /* need to cleanup */
-  g_spawn_close_pid (pid);
+  thunar_uca_provider_child_watch_destroy (uca_provider, NULL);
 
   GDK_THREADS_LEAVE ();
 }
@@ -451,15 +459,27 @@ thunar_uca_provider_child_watch (GPid     pid,
 
 
 static void
-thunar_uca_provider_child_watch_destroy (gpointer user_data)
+thunar_uca_provider_child_watch_destroy (gpointer  user_data,
+                                         GClosure *closure)
 {
   ThunarUcaProvider *uca_provider = THUNAR_UCA_PROVIDER (user_data);
+  GClosure          *child_watch;
 
-  /* reset child watch id and path */
-  g_free (uca_provider->child_watch_path);
-  uca_provider->child_watch_path = NULL;
-  uca_provider->child_watch_id = -1;
+  /* leave if the closure is not the one we're watching */
+  if (uca_provider->child_watch == closure
+      || closure == NULL)
+    {
+      /* reset child watch and path */
+      if (G_UNLIKELY (uca_provider->child_watch != NULL))
+        {
+          child_watch = uca_provider->child_watch;
+          uca_provider->child_watch = NULL;
+
+          g_closure_invalidate (child_watch);
+          g_closure_unref (child_watch);
+        }
+
+      g_free (uca_provider->child_watch_path);
+      uca_provider->child_watch_path = NULL;
+    }
 }
-
-
-
