@@ -44,6 +44,7 @@
 
 #include <thunar/thunar-application.h>
 #include <thunar/thunar-browser.h>
+#include <thunar/thunar-create-dialog.h>
 #include <thunar/thunar-dialogs.h>
 #include <thunar/thunar-gdk-extensions.h>
 #include <thunar/thunar-gobject-extensions.h>
@@ -148,6 +149,7 @@ struct _ThunarApplication
 
 static GQuark thunar_application_screen_quark;
 static GQuark thunar_application_startup_id_quark;
+static GQuark thunar_application_file_quark;
 
 
 
@@ -166,6 +168,8 @@ thunar_application_class_init (ThunarApplicationClass *klass)
     g_quark_from_static_string ("thunar-application-screen");
   thunar_application_startup_id_quark =
     g_quark_from_static_string ("thunar-application-startup-id");
+  thunar_application_file_quark =
+    g_quark_from_static_string ("thunar-application-file");
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_application_finalize;
@@ -1209,11 +1213,217 @@ thunar_application_is_processing (ThunarApplication *application)
 
 
 
+static void
+thunar_application_rename_file_error (ExoJob            *job,
+                                      GError            *error,
+                                      ThunarApplication *application)
+{
+  ThunarFile *file;
+  GdkScreen  *screen;
+
+  _thunar_return_if_fail (EXO_IS_JOB (job));
+  _thunar_return_if_fail (error != NULL);
+  _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
+
+  screen = g_object_get_qdata (G_OBJECT (job), thunar_application_screen_quark);
+  file = g_object_get_qdata (G_OBJECT (job), thunar_application_file_quark);
+
+  g_assert (screen != NULL);
+  g_assert (file != NULL);
+
+  thunar_dialogs_show_error (screen, error, _("Failed to rename \"%s\""), 
+                             thunar_file_get_display_name (file));
+}
+
+
+
+/**
+ * thunar_application_rename_file:
+ * @application : a #ThunarApplication.
+ * @file        : a #ThunarFile to be renamed.
+ * @screen      : the #GdkScreen on which to open the window or %NULL
+ *                to open on the default screen.
+ * @startup_id  : startup id from startup notification passed along
+ *                with dbus to make focus stealing work properly.
+ *
+ * Prompts the user to rename the @file.
+ **/
+void
+thunar_application_rename_file (ThunarApplication *application,
+                                ThunarFile        *file,
+                                GdkScreen         *screen,
+                                const gchar       *startup_id)
+{
+  ThunarJob *job;
+
+  _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
+  _thunar_return_if_fail (THUNAR_IS_FILE (file));
+  _thunar_return_if_fail (GDK_IS_SCREEN (screen));
+  _thunar_return_if_fail (startup_id != NULL);
+
+  /* TODO pass the startup ID to the rename dialog */
+
+  /* run the rename dialog */
+  job = thunar_dialogs_show_rename_file (screen, file);
+  if (G_LIKELY (job != NULL))
+    {
+      /* remember the screen and file */
+      g_object_set_qdata (G_OBJECT (job), thunar_application_screen_quark, screen);
+      g_object_set_qdata_full (G_OBJECT (job), thunar_application_file_quark, 
+                               g_object_ref (file), g_object_unref);
+
+      /* handle rename errors */
+      g_signal_connect (job, "error", 
+                        G_CALLBACK (thunar_application_rename_file_error), application);
+    }
+}
+
+
+
+/**
+ * thunar_application_create_file:
+ * @application      : a #ThunarApplication.
+ * @parent_directory : the #ThunarFile of the parent directory.
+ * @content_type     : the content type of the new file.
+ * @screen           : the #GdkScreen on which to open the window or %NULL
+ *                     to open on the default screen.
+ * @startup_id       : startup id from startup notification passed along
+ *                     with dbus to make focus stealing work properly.
+ *
+ * Prompts the user to create a new file or directory in @parent_directory.
+ * The @content_type defines the icon and other elements in the filename 
+ * prompt dialog.
+ **/
+void
+thunar_application_create_file (ThunarApplication *application,
+                                ThunarFile        *parent_directory,
+                                const gchar       *content_type,
+                                GdkScreen         *screen,
+                                const gchar       *startup_id)
+{
+  const gchar *dialog_title;
+  const gchar *title;
+  gboolean     is_directory;
+  GList        path_list;
+  gchar       *name;
+
+  _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
+  _thunar_return_if_fail (THUNAR_IS_FILE (parent_directory));
+  _thunar_return_if_fail (content_type != NULL && *content_type != '\0');
+  _thunar_return_if_fail (GDK_IS_SCREEN (screen));
+  _thunar_return_if_fail (startup_id != NULL);
+
+  is_directory = (g_strcmp0 (content_type, "inode/directory") == 0);
+
+  if (is_directory)
+    {
+      dialog_title = _("New Folder");
+      title = _("Create New Folder");
+    }
+  else
+    {
+      dialog_title = _("New File");
+      title = _("Create New File");
+    }
+
+  /* TODO pass the startup ID to the rename dialog */
+
+  /* ask the user to enter a name for the new folder */
+  name = thunar_show_create_dialog (screen, content_type, dialog_title, title);
+  if (G_LIKELY (name != NULL))
+    {
+      path_list.data = g_file_get_child (thunar_file_get_file (parent_directory), name);
+      path_list.next = path_list.prev = NULL;
+
+      /* launch the operation */
+      if (is_directory)
+        thunar_application_mkdir (application, screen, &path_list, NULL);
+      else
+        thunar_application_creat (application, screen, &path_list, NULL);
+
+      g_object_unref (path_list.data);
+      g_free (name);
+    }
+}
+
+
+
+/**
+ * thunar_application_create_file_from_template:
+ * @application      : a #ThunarApplication.
+ * @parent_directory : the #ThunarFile of the parent directory.
+ * @template_file    : the #ThunarFile of the template.
+ * @screen           : the #GdkScreen on which to open the window or %NULL
+ *                     to open on the default screen.
+ * @startup_id       : startup id from startup notification passed along
+ *                     with dbus to make focus stealing work properly.
+ *
+ * Prompts the user to create a new file or directory in @parent_directory
+ * from an existing @template_file which predefines the name and extension
+ * in the create dialog.
+ **/
+void
+thunar_application_create_file_from_template (ThunarApplication *application,
+                                              ThunarFile        *parent_directory,
+                                              ThunarFile        *template_file,
+                                              GdkScreen         *screen,
+                                              const gchar       *startup_id)
+{
+  GList  source_path_list;
+  GList  target_path_list;
+  gchar *name;
+  gchar *title;
+
+  _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
+  _thunar_return_if_fail (THUNAR_IS_FILE (parent_directory));
+  _thunar_return_if_fail (THUNAR_IS_FILE (template_file));
+  _thunar_return_if_fail (GDK_IS_SCREEN (screen));
+  _thunar_return_if_fail (startup_id != NULL);
+
+  /* generate a title for the create dialog */
+  title = g_strdup_printf (_("Create Document from template \"%s\""),
+                           thunar_file_get_display_name (template_file));
+
+  /* TODO pass the startup ID to the rename dialog */
+
+  /* ask the user to enter a name for the new document */
+  name = thunar_show_create_dialog (screen, 
+                                    thunar_file_get_content_type (template_file),
+                                    thunar_file_get_display_name (template_file), 
+                                    title);
+  if (G_LIKELY (name != NULL))
+    {
+      /* fake the source file list */
+      source_path_list.data = thunar_file_get_file (template_file);
+      source_path_list.prev = source_path_list.next = NULL;
+
+      /* fake the target path list */
+      target_path_list.data = g_file_get_child (thunar_file_get_file (parent_directory), name);
+      target_path_list.next = target_path_list.prev = NULL;
+
+      /* launch the operation */
+      thunar_application_copy_to (application, screen, 
+                                  &source_path_list, &target_path_list,
+                                  NULL);
+
+      /* release the target path */
+      g_object_unref (target_path_list.data);
+
+      /* release the file name */
+      g_free (name);
+    }
+
+  /* clean up */
+  g_free (title);
+}
+
+
+
 /**
  * thunar_application_copy_to:
  * @application       : a #ThunarApplication.
  * @parent            : a #GdkScreen, a #GtkWidget or %NULL.
- * @source_file_list  : the list of #GFile<!---->s that should be copied.
+ * @source_file_list  : the lst of #GFile<!---->s that should be copied.
  * @target_file_list  : the list of #GFile<!---->s where files should be copied to.
  * @new_files_closure : a #GClosure to connect to the job's "new-files" signal,
  *                      which will be emitted when the job finishes with the
